@@ -290,6 +290,11 @@ class XFormToDictBuilder:
             question['__order'] = self._get_question_order(ref)
             if 'calculate' in item:
                 question['type'] = 'calculate'
+            if ref.split('/').__len__() == 3:
+                # just append on root node, has no group
+                question['ref'] = ref
+                self.children.append(question)
+                continue
             for child in self.children:
                 if child['ref'] == parent_ref:
                     question['ref'] = ref
@@ -305,6 +310,8 @@ class XFormToDictBuilder:
                 new_ref = u'/'.join(ref.split('/')[2:])
                 root_ref = u'/'.join(ref.split('/')[:2])
                 q = self._get_item_func(root_ref, new_ref, item)
+                if 'type' not in q and 'type' in question:
+                    q.update(question)
                 if q['type'] == 'group' and q['name'] == 'meta':
                     q['control'] = {'bodyless': True}
                     q['__order'] = self._get_question_order(ref)
@@ -445,7 +452,12 @@ class XFormToDictBuilder:
                     if k == 'type':
                         v = self._get_question_type(v)
                     if k in ['relevant', 'required', 'constraint',
-                             'constraintMsg', 'readonly', 'calculate']:
+                             'constraintMsg', 'readonly', 'calculate',
+                             'noAppErrorString', 'requiredMsg']:
+                        if k == 'noAppErrorString':
+                            k = 'jr:noAppErrorString'
+                        if k == 'requiredMsg':
+                            k = 'jr:requiredMsg'
                         if k == 'constraintMsg':
                             k = "jr:constraintMsg"
                             v = self._get_constraintMsg(v)
@@ -587,12 +599,11 @@ class XFormToDictBuilder:
 
         def replace_function(match):
             return "${%s}" % get_last_item(match.group())
-        pattern = "( /[a-z0-9\-_]+(?:/[a-z0-9\-_ ]+)+)"
-        text = re.sub(
-            pattern, replace_function, text, flags=re.I)
+        #moving re flags into compile for python 2.6 compat
+        pattern = "( /[a-z0-9\-_]+(?:/[a-z0-9\-_]+)+ )"
+        text = re.compile(pattern, flags=re.I).sub(replace_function, text)
         pattern = "(/[a-z0-9\-_]+(?:/[a-z0-9\-_]+)+)"
-        text = re.sub(
-            pattern, replace_function, text, flags=re.I)
+        text = re.compile(pattern, flags=re.I).sub(replace_function, text)
         return text
 
 
